@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Alexandre Lision <alexandre.lision@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -30,9 +30,14 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import cx.ring.smartlist.SmartListViewModel;
 import cx.ring.utils.Log;
 import cx.ring.utils.StringUtils;
+import cx.ring.utils.Tuple;
+import ezvcard.VCard;
+import ezvcard.property.FormattedName;
 import io.reactivex.Observable;
+import io.reactivex.Single;
 import io.reactivex.subjects.BehaviorSubject;
 import io.reactivex.subjects.PublishSubject;
 import io.reactivex.subjects.Subject;
@@ -49,11 +54,11 @@ public class Account {
 
     private AccountConfig mVolatileDetails;
     private AccountConfig mDetails;
-    private ArrayList<AccountCredentials> credentialsDetails = new ArrayList<>();
+    private final ArrayList<AccountCredentials> credentialsDetails = new ArrayList<>();
     private Map<String, String> devices = new HashMap<>();
-    private Map<String, CallContact> mContacts = new HashMap<>();
-    private Map<String, TrustRequest> mRequests = new HashMap<>();
-    private Map<String, CallContact> mContactCache = new HashMap<>();
+    private final Map<String, CallContact> mContacts = new HashMap<>();
+    private final Map<String, TrustRequest> mRequests = new HashMap<>();
+    private final Map<String, CallContact> mContactCache = new HashMap<>();
 
     private final Map<String, Conversation> conversations = new HashMap<>();
     private final Map<String, Conversation> pending = new HashMap<>();
@@ -62,83 +67,67 @@ public class Account {
     private final List<Conversation> sortedConversations = new ArrayList<>();
     private final List<Conversation> sortedPending = new ArrayList<>();
 
-    private final Subject<Conversation> conversationSubject = PublishSubject.create();
-    private final Subject<List<Conversation>> conversationsSubject = BehaviorSubject.create();
-    private final Subject<List<Conversation>> pendingSubject = BehaviorSubject.create();
-
     public boolean registeringUsername = false;
     private boolean conversationsChanged = true;
     private boolean pendingsChanged = true;
     private boolean historyLoaded = false;
 
+    private final Subject<Conversation> conversationSubject = PublishSubject.create();
+    private final Subject<List<Conversation>> conversationsSubject = BehaviorSubject.create();
+    private final Subject<List<Conversation>> pendingSubject = BehaviorSubject.create();
+
     private final BehaviorSubject<Collection<CallContact>> contactListSubject = BehaviorSubject.create();
     private final BehaviorSubject<Collection<TrustRequest>> trustRequestsSubject = BehaviorSubject.create();
-    private final Subject<RequestEvent> trustRequestSubject = PublishSubject.create();
+    public Subject<Account> historyLoader;
+    private VCard mProfile;
+    private Single<Tuple<String, Object>> mLoadedProfile = null;
 
+    public Account(String bAccountID) {
+        accountID = bAccountID;
+        mDetails = new AccountConfig();
+        mVolatileDetails = new AccountConfig();
+    }
+    public Account(String bAccountID, final Map<String, String> details,
+                   final List<Map<String, String>> credentials,
+                   final Map<String, String> volDetails) {
+        accountID = bAccountID;
+        mDetails = new AccountConfig(details);
+        mVolatileDetails = new AccountConfig(volDetails);
+        setCredentials(credentials);
+    }
+
+    public void cleanup() {
+        conversationSubject.onComplete();
+        conversationsSubject.onComplete();
+        pendingSubject.onComplete();
+        contactListSubject.onComplete();
+        trustRequestsSubject.onComplete();
+    }
 
     public Observable<List<Conversation>> getConversationsSubject() {
         return conversationsSubject;
     }
+    public Observable<List<SmartListViewModel>> getConversationsViewModels() {
+        return conversationsSubject
+                .map(conversations -> {
+                    ArrayList<SmartListViewModel> viewModel = new ArrayList<>(conversations.size());
+                    for (Conversation c : conversations)
+                        viewModel.add(new SmartListViewModel(accountID, c.getContact(), c.getLastEvent()));
+                    return viewModel;
+                });
+    }
+
     public Observable<Conversation> getConversationSubject() {
         return conversationSubject;
     }
+    public Observable<SmartListViewModel> getConversationViewModel() {
+        return conversationSubject.map(c -> new SmartListViewModel(accountID, c.getContact(), c.getLastEvent()));
+    }
+
     public Observable<List<Conversation>> getPendingSubject() {
         return pendingSubject;
     }
 
-    public Subject<Account> historyLoader;
-
-    private static class ConversationComparator implements Comparator<Conversation> {
-        @Override
-        public int compare(Conversation a, Conversation b) {
-            ConversationElement eventA = a.getLastEvent();
-            ConversationElement eventB = b.getLastEvent();
-            if (eventA == null) {
-                if (eventB == null)
-                    return 0;
-                return 1;
-            }
-            if (eventB == null)
-                return -1;
-            return Long.compare(eventB.getDate(), eventA.getDate());
-        }
-    }
-
-    private static final ConversationComparator CONVERSATION_COMPARATOR = new ConversationComparator();
-
-    private List<Conversation> getSortedConversations() {
-        Log.w(TAG, "getSortedConversations() " + Thread.currentThread().getId());
-        synchronized (sortedConversations) {
-            if (conversationsChanged) {
-                sortedConversations.clear();
-                synchronized (conversations) {
-                    sortedConversations.addAll(conversations.values());
-                }
-                for (Conversation c : sortedConversations)
-                    c.sortHistory();
-                Collections.sort(sortedConversations, CONVERSATION_COMPARATOR);
-                conversationsChanged = false;
-            }
-        }
-        return sortedConversations;
-    }
-
-    private List<Conversation> getSortedPending() {
-        synchronized (sortedPending) {
-            if (pendingsChanged) {
-                sortedPending.clear();
-                synchronized (pending) {
-                    sortedPending.addAll(pending.values());
-                }
-                for (Conversation c : sortedPending)
-                    c.sortHistory();
-                Collections.sort(sortedPending, CONVERSATION_COMPARATOR);
-                pendingsChanged = false;
-            }
-        }
-        return sortedPending;
-    }
-
     public Collection<Conversation> getConversations() {
         return conversations.values();
     }
@@ -147,11 +136,11 @@ public class Account {
         return pending.values();
     }
 
-    public void pendingRefreshed() {
+    private void pendingRefreshed() {
         if (historyLoaded)
             pendingSubject.onNext(getSortedPending());
     }
-    public void pendingChanged() {
+    private void pendingChanged() {
         pendingsChanged = true;
         pendingRefreshed();
     }
@@ -163,19 +152,19 @@ public class Account {
         } else {
             if (conversation != null)
                 conversation.sortHistory();
-            Collections.sort(sortedPending, (a, b) -> Long.compare(b.getLastEvent().getDate(), a.getLastEvent().getDate()));
+            Collections.sort(sortedPending, (a, b) -> ConversationElement.compare(b.getLastEvent(), a.getLastEvent()));
         }
         pendingSubject.onNext(getSortedPending());
     }
 
-    public void conversationRefreshed(Conversation conversation) {
+    private void conversationRefreshed(Conversation conversation) {
         if (historyLoaded)
             conversationSubject.onNext(conversation);
     }
-    public void conversationChanged() {
+    private void conversationChanged() {
         conversationsChanged = true;
         if (historyLoaded)
-            conversationsSubject.onNext(getSortedConversations());
+            conversationsSubject.onNext(new ArrayList<>(getSortedConversations()));
     }
     public void conversationUpdated(Conversation conversation) {
         if (!historyLoaded)
@@ -186,9 +175,9 @@ public class Account {
             } else {
                 if (conversation != null)
                     conversation.sortHistory();
-                Collections.sort(sortedConversations, (a, b) -> Long.compare(b.getLastEvent().getDate(), a.getLastEvent().getDate()));
+                Collections.sort(sortedConversations, (a, b) -> ConversationElement.compare(b.getLastEvent(), a.getLastEvent()));
             }
-            conversationsSubject.onNext(sortedConversations);
+            conversationsSubject.onNext(new ArrayList<>(sortedConversations));
         }
     }
 
@@ -205,8 +194,13 @@ public class Account {
         else if (conversation == pending.get(key))
             pendingUpdated(conversation);
         else if (conversation == cache.get(key)) {
-            pending.put(key, conversation);
-            pendingChanged();
+            if (isRing()) {
+                pending.put(key, conversation);
+                pendingChanged();
+            } else {
+                conversations.put(key, conversation);
+                conversationChanged();
+            }
         }
     }
 
@@ -218,10 +212,11 @@ public class Account {
     }
 
     public void addTextMessage(TextMessage txt) {
-        Conversation conversation;
+        Conversation conversation = null;
         if (!StringUtils.isEmpty(txt.getCallId())) {
             conversation = getConversationByCallId(txt.getCallId());
-        } else {
+        }
+        if (conversation == null) {
             conversation = getByUri(txt.getNumberUri());
             txt.setContact(conversation.getContact());
         }
@@ -241,80 +236,32 @@ public class Account {
         return conversation;
     }
 
-    public static class ContactEvent {
-        public final CallContact contact;
-        public final boolean added;
-        ContactEvent(CallContact c, boolean a) { contact=c; added=a;}
-    }
-    public static class RequestEvent {
-        public final TrustRequest request;
-        public final boolean added;
-        RequestEvent(TrustRequest c, boolean a) { request=c; added=a;}
-    }
-
-    private final Observable<CallContact> contactsObservable = Observable.create(subscriber -> {
-        for (CallContact c : mContacts.values())
-            subscriber.onNext(c);
-        subscriber.onComplete();
-    });
-    private final Observable<CallContact> validContactsObservable = contactsObservable.filter(c -> !c.isBanned());
-    private final Observable<CallContact> bannedContactsObservable = contactsObservable.filter(CallContact::isBanned);
-
-    public Observable<CallContact> getValidContacts() {
-        return validContactsObservable;
-    }
-
-    public Observable<Collection<CallContact>> getContactsUpdates() {
-        return contactListSubject;
-    }
-    public Observable<Collection<TrustRequest>> getRequestsUpdates() {
-        return trustRequestsSubject;
-    }
-    public Observable<RequestEvent> getRequestsEvents() {
-        return trustRequestSubject;
-    }
-    public Observable<Collection<CallContact>> getValidContactsUpdates() {
-        return contactListSubject
-                .concatMapSingle(list -> Observable.fromIterable(list)
-                        .filter(c -> !c.isBanned()).toList());
-    }
     public Observable<Collection<CallContact>> getBannedContactsUpdates() {
         return contactListSubject.concatMapSingle(list -> Observable.fromIterable(list).filter(CallContact::isBanned).toList());
     }
 
     public CallContact getContactFromCache(String key) {
-        CallContact contact = mContactCache.get(key);
-        if (contact == null) {
-            contact = CallContact.buildUnknown(key);
-            mContactCache.put(key, contact);
+        synchronized (mContactCache) {
+            CallContact contact = mContactCache.get(key);
+            if (contact == null) {
+                if (isSip())
+                    contact = CallContact.buildSIP(new Uri(key));
+                else
+                    contact = CallContact.build(key);
+                mContactCache.put(key, contact);
+            }
+            return contact;
         }
-        return contact;
     }
     public CallContact getContactFromCache(Uri uri) {
         return getContactFromCache(uri.getRawUriString());
     }
 
-    public Account(String bAccountID) {
-        accountID = bAccountID;
-        mDetails = new AccountConfig();
-        mVolatileDetails = new AccountConfig();
-    }
-
     public void dispose() {
         contactListSubject.onComplete();
-        //contactSubject.onComplete();
         trustRequestsSubject.onComplete();
     }
 
-    public Account(String bAccountID, final Map<String, String> details,
-                   final List<Map<String, String>> credentials,
-                   final Map<String, String> volDetails) {
-        accountID = bAccountID;
-        mDetails = new AccountConfig(details);
-        mVolatileDetails = new AccountConfig(volDetails);
-        setCredentials(credentials);
-    }
-
     public Map<String, String> getDevices() {
         return devices;
     }
@@ -431,7 +378,7 @@ public class Account {
         mDetails.put(ConfigKey.ACCOUNT_ALIAS, alias);
     }
 
-    public String getDetail(ConfigKey key) {
+    private String getDetail(ConfigKey key) {
         return mDetails.get(key);
     }
 
@@ -451,6 +398,14 @@ public class Account {
         mDetails.put(ConfigKey.ACCOUNT_ENABLE, isChecked);
     }
 
+    public boolean hasPassword() {
+        return mDetails.getBool(ConfigKey.ARCHIVE_HAS_PASSWORD);
+    }
+
+    public boolean hasManager() {
+        return !mDetails.get(ConfigKey.MANAGER_URI).isEmpty();
+    }
+
     public HashMap<String, String> getDetails() {
         return mDetails.getAll();
     }
@@ -510,7 +465,7 @@ public class Account {
         return mDetails.getBool(ConfigKey.SRTP_ENABLE) || mDetails.getBool(ConfigKey.TLS_ENABLE);
     }
 
-    public String getUri(boolean display) {
+    private String getUri(boolean display) {
         String username = display ? getDisplayUsername() : getUsername();
         if (isRing()) {
             return username;
@@ -526,6 +481,9 @@ public class Account {
     public String getDisplayUri() {
         return getUri(true);
     }
+    public String getDisplayUri(CharSequence defaultNameSip) {
+        return isIP2IP() ? defaultNameSip.toString() : getDisplayUri();
+    }
 
     public boolean needsMigration() {
         return AccountConfig.STATE_NEED_MIGRATION.equals(getRegistrationState());
@@ -625,12 +583,10 @@ public class Account {
     }
 
     public void setContacts(List<Map<String, String>> contacts) {
-        Log.w(TAG, "setContacts");
         for (Map<String, String> contact : contacts) {
             addContact(contact);
         }
         contactListSubject.onNext(mContacts.values());
-        conversationChanged();
     }
 
     public List<TrustRequest> getRequests() {
@@ -643,18 +599,14 @@ public class Account {
         return requests;
     }
 
-    public TrustRequest getRequest(String contactId) {
-        return mRequests.get(contactId);
+    public TrustRequest getRequest(Uri uri) {
+        return mRequests.get(uri.getRawUriString());
     }
 
     public void addRequest(TrustRequest request) {
         synchronized (pending) {
             mRequests.put(request.getContactId(), request);
-            CallContact contact = getContactFromCache(request.getContactId());
-            if (contact.vcard == null) {
-                contact.vcard = request.getVCard();
-            }
-            trustRequestSubject.onNext(new RequestEvent(request, true));
+            //trustRequestSubject.onNext(new RequestEvent(request, true));
             trustRequestsSubject.onNext(mRequests.values());
 
             String key = new Uri(request.getContactId()).getRawUriString();
@@ -680,7 +632,7 @@ public class Account {
                     pending.put(key, conversation);
                     conversation.addRequestEvent(request);
                 }
-                trustRequestSubject.onNext(new RequestEvent(request, true));
+                //trustRequestSubject.onNext(new RequestEvent(request, true));
             }
             trustRequestsSubject.onNext(mRequests.values());
             pendingChanged();
@@ -691,7 +643,7 @@ public class Account {
         synchronized (pending) {
             TrustRequest request = mRequests.remove(contact.getRawUriString());
             if (request != null) {
-                trustRequestSubject.onNext(new RequestEvent(request, true));
+                //trustRequestSubject.onNext(new RequestEvent(request, true));
                 trustRequestsSubject.onNext(mRequests.values());
             }
             if (pending.remove(contact.getRawUriString()) != null) {
@@ -703,22 +655,18 @@ public class Account {
     }
 
     public void registeredNameFound(int state, String address, String name) {
-        if (state == 0) {
-            Log.w(TAG, "registeredNameFound " + address + " " + name);
-            Uri uri = new Uri(address);
-            CallContact contact = getContactFromCache(uri);
-            if (contact.setUsername(name)) {
-                Log.w(TAG, "username set " + name);
-                String key = uri.getRawUriString();
-                synchronized (conversations) {
-                    Conversation conversation = conversations.get(key);
-                    if (conversation != null)
-                        conversationRefreshed(conversation);
-                }
-                synchronized (pending) {
-                    if (pending.containsKey(key))
-                        pendingRefreshed();
-                }
+        Uri uri = new Uri(address);
+        CallContact contact = getContactFromCache(uri);
+        if (contact.setUsername(state == 0 ? name : null)) {
+            String key = uri.getRawUriString();
+            synchronized (conversations) {
+                Conversation conversation = conversations.get(key);
+                if (conversation != null)
+                    conversationRefreshed(conversation);
+            }
+            synchronized (pending) {
+                if (pending.containsKey(key))
+                    pendingRefreshed();
             }
         }
     }
@@ -730,51 +678,6 @@ public class Account {
         return null;
     }
 
-    public Conversation getByKey(String key) {
-        Conversation conversation = cache.get(key);
-        if (conversation != null) {
-            return conversation;
-        }
-        CallContact contact = getContactFromCache(key);
-        conversation = new Conversation(contact);
-        cache.put(key, conversation);
-        return conversation;
-    }
-
-    public void contactAdded(CallContact contact) {
-        Uri uri = contact.getPrimaryUri();
-        String key = uri.getRawUriString();
-        synchronized (conversations) {
-            if (conversations.get(key) != null)
-                return;
-            synchronized (pending) {
-                Conversation pendingConversation = pending.get(key);
-                if (pendingConversation == null) {
-                    pendingConversation = getByKey(key);
-                    conversations.put(key, pendingConversation);
-                } else {
-                    pending.remove(key);
-                    conversations.put(key, pendingConversation);
-                    pendingChanged();
-                }
-                pendingConversation.addContactEvent();
-            }
-            conversationChanged();
-        }
-    }
-
-    public void contactRemoved(Uri uri) {
-        String key = uri.getRawUriString();
-        synchronized (conversations) {
-            synchronized (pending) {
-                if (pending.remove(key) != null)
-                    pendingChanged();
-            }
-            conversations.remove(key);
-            conversationChanged();
-        }
-    }
-
     public boolean isHistoryLoaded() {
         return historyLoaded;
     }
@@ -794,7 +697,85 @@ public class Account {
         pendingChanged();
     }
 
-    public Conversation getConversationByCallId(String callId) {
+    private List<Conversation> getSortedConversations() {
+        Log.w(TAG, "getSortedConversations() " + Thread.currentThread().getId());
+        synchronized (sortedConversations) {
+            if (conversationsChanged) {
+                sortedConversations.clear();
+                synchronized (conversations) {
+                    sortedConversations.addAll(conversations.values());
+                }
+                for (Conversation c : sortedConversations)
+                    c.sortHistory();
+                Collections.sort(sortedConversations, new ConversationComparator());
+                conversationsChanged = false;
+            }
+        }
+        return sortedConversations;
+    }
+
+    private List<Conversation> getSortedPending() {
+        synchronized (sortedPending) {
+            if (pendingsChanged) {
+                sortedPending.clear();
+                synchronized (pending) {
+                    sortedPending.addAll(pending.values());
+                }
+                for (Conversation c : sortedPending)
+                    c.sortHistory();
+                Collections.sort(sortedPending, new ConversationComparator());
+                pendingsChanged = false;
+            }
+        }
+        return sortedPending;
+    }
+
+    private Conversation getByKey(String key) {
+        Conversation conversation = cache.get(key);
+        if (conversation != null) {
+            return conversation;
+        }
+        CallContact contact = getContactFromCache(key);
+        conversation = new Conversation(getAccountID(), contact);
+        cache.put(key, conversation);
+        return conversation;
+    }
+
+    private void contactAdded(CallContact contact) {
+        Uri uri = contact.getPrimaryUri();
+        String key = uri.getRawUriString();
+        synchronized (conversations) {
+            if (conversations.get(key) != null)
+                return;
+            synchronized (pending) {
+                Conversation pendingConversation = pending.get(key);
+                if (pendingConversation == null) {
+                    pendingConversation = getByKey(key);
+                    conversations.put(key, pendingConversation);
+                } else {
+                    pending.remove(key);
+                    conversations.put(key, pendingConversation);
+                    pendingChanged();
+                }
+                pendingConversation.addContactEvent();
+            }
+            conversationChanged();
+        }
+    }
+
+    private void contactRemoved(Uri uri) {
+        String key = uri.getRawUriString();
+        synchronized (conversations) {
+            synchronized (pending) {
+                if (pending.remove(key) != null)
+                    pendingChanged();
+            }
+            conversations.remove(key);
+            conversationChanged();
+        }
+    }
+
+    private Conversation getConversationByCallId(String callId) {
         for (Conversation conversation : conversations.values()) {
             Conference conf = conversation.getConference(callId);
             if (conf != null) {
@@ -804,4 +785,52 @@ public class Account {
         return null;
     }
 
+    public void presenceUpdate(String contactUri, boolean isOnline) {
+        String key = CallContact.PREFIX_RING + contactUri;
+        CallContact contact = getContactFromCache(key);
+        if (contact.isOnline() == isOnline)
+            return;
+        contact.setOnline(isOnline);
+        synchronized (conversations) {
+            Conversation conversation = conversations.get(key);
+            if (conversation != null) {
+                conversationRefreshed(conversation);
+            }
+        }
+        synchronized (pending) {
+            if (pending.containsKey(key))
+                pendingRefreshed();
+        }
+    }
+
+    public Single<String> getAccountAlias() {
+        if (mLoadedProfile == null)
+            return Single.just(getAlias());
+        return mLoadedProfile.map(p -> StringUtils.isEmpty(p.first) ? getAlias() : p.first);
+    }
+
+    public void setProfile(VCard vcard) {
+        mProfile = vcard;
+        mLoadedProfile = null;
+    }
+
+    public VCard getProfile() {
+        return mProfile;
+    }
+
+    public Single<Tuple<String, Object>> getLoadedProfile() {
+        return mLoadedProfile;
+    }
+
+    public void setLoadedProfile(Single<Tuple<String, Object>> profile) {
+        mLoadedProfile = profile;
+    }
+
+    private static class ConversationComparator implements Comparator<Conversation> {
+        @Override
+        public int compare(Conversation a, Conversation b) {
+            return ConversationElement.compare(b.getLastEvent(), a.getLastEvent());
+        }
+    }
+
 }
