@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2016-2019 Savoir-faire Linux Inc.
  *  Author: Alexandre Viau <alexandre.viau@savoirfairelinux.com>
  *  Author: Nicolas Jäger <nicolas.jager@savoirfairelinux.com>
  *
@@ -60,13 +60,17 @@ struct _AccountMigrationViewPrivate
     /* main_view */
     GtkWidget *main_view;
     GtkWidget *label_account_alias;
-    GtkWidget *label_account_ringid;
+    GtkWidget *label_account_username;
+    GtkWidget *label_account_manager;
     GtkWidget *image_avatar;
     GtkWidget *label_migration_error;
     GtkWidget *entry_password;
     GtkWidget *button_migrate_account;
-    GtkWidget *ringid_row;
+    GtkWidget *username_row;
+    GtkWidget *manager_row;
     GtkWidget *button_delete_account;
+
+    GtkWidget *hbox_migrating_account_spinner;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE(AccountMigrationView, account_migration_view, GTK_TYPE_BOX);
@@ -105,17 +109,20 @@ account_migration_view_class_init(AccountMigrationViewClass *klass)
     G_OBJECT_CLASS(klass)->dispose = account_migration_view_dispose;
 
     gtk_widget_class_set_template_from_resource(GTK_WIDGET_CLASS (klass),
-                                                "/cx/jami/JamiGnome/accountmigrationview.ui");
+                                                "/net/jami/JamiGnome/accountmigrationview.ui");
     /* main_view */
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, main_view);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, label_account_alias);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, label_account_ringid);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, label_account_username);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, label_account_manager);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, image_avatar);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, label_migration_error);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, entry_password);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, button_migrate_account);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, ringid_row);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, username_row);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, manager_row);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, button_delete_account);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountMigrationView, hbox_migrating_account_spinner);
 
     /* add signals */
     account_migration_view_signals[ACCOUNT_MIGRATION_COMPLETED] = g_signal_new("account-migration-completed",
@@ -147,10 +154,9 @@ migrate(AccountMigrationView *view)
                                            &lrc::api::NewAccountModel::migrationEnded,
         [=] (const std::string& accountId, bool ok)
         {
-            if ((*priv->accountInfo_)->id != accountId) {
-                return;
-            }
-
+            gtk_widget_hide(priv->hbox_migrating_account_spinner);
+            gtk_widget_set_sensitive(GTK_WIDGET(priv->button_delete_account), true);
+            gtk_widget_set_sensitive(GTK_WIDGET(priv->button_migrate_account), true);
             if (ok) {
                 g_signal_emit(G_OBJECT(view), account_migration_view_signals[ACCOUNT_MIGRATION_COMPLETED], 0);
             } else {
@@ -162,6 +168,10 @@ migrate(AccountMigrationView *view)
     currentProp.archivePassword = password;
     (*priv->accountInfo_)->accountModel->setAccountConfig((*priv->accountInfo_)->id, currentProp);
     gtk_entry_set_text(GTK_ENTRY(priv->entry_password), "");
+
+    gtk_widget_show_all(priv->hbox_migrating_account_spinner);
+    gtk_widget_set_sensitive(GTK_WIDGET(priv->button_delete_account), false);
+    gtk_widget_set_sensitive(GTK_WIDGET(priv->button_migrate_account), false);
 }
 
 
@@ -209,13 +219,24 @@ build_migration_view(AccountMigrationView *view)
 
     // display the ringID (without "ring:")
     g_debug("MIGRATE FOR %s", (*priv->accountInfo_)->id.c_str());
-    std::string username = (*priv->accountInfo_)->profileInfo.uri;
-    if (username.empty()) {
-        username = (*priv->accountInfo_)->profileInfo.uri;
-    }
-    gtk_label_set_text(GTK_LABEL(priv->label_account_ringid), username.c_str());
-    if (username.empty()) {
-        gtk_widget_hide(priv->ringid_row);
+    std::string username = (*priv->accountInfo_)->registeredName;
+    try {
+        auto conf = (*priv->accountInfo_)->accountModel->getAccountConfig((*priv->accountInfo_)->id);
+        if (username.empty() && !conf.managerUsername.empty()) {
+            username = conf.managerUsername;
+        }
+        gtk_label_set_text(GTK_LABEL(priv->label_account_username), username.c_str());
+        if (username.empty()) {
+            gtk_widget_hide(priv->username_row);
+        }
+        std::string manager = conf.managerUri;
+        gtk_label_set_text(GTK_LABEL(priv->label_account_manager), manager.c_str());
+        if (manager.empty()) {
+            gtk_widget_hide(priv->manager_row);
+        }
+    } catch (...) {
+        gtk_widget_hide(priv->username_row);
+        gtk_widget_hide(priv->manager_row);
     }
 
     /* get the current or default profile avatar */
@@ -244,6 +265,8 @@ build_migration_view(AccountMigrationView *view)
     GtkStyleContext* context;
     context = gtk_widget_get_style_context(GTK_WIDGET(priv->button_delete_account));
     gtk_style_context_add_class(context, "button_red");
+
+    gtk_widget_hide(priv->hbox_migrating_account_spinner);
 }
 
 GtkWidget *
