@@ -200,6 +200,8 @@ typedef NS_ENUM(NSInteger, ViewState) {
     if(appSandboxed()) {
         NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
         avModel->setRecordPath([[paths objectAtIndex:0] UTF8String]);
+    } else if (avModel->getRecordPath().empty()) {
+        avModel->setRecordPath([NSHomeDirectory() UTF8String]);
     }
     NSToolbar *tb = [[self window] toolbar];
     [tb setAllowsUserCustomization:NO];
@@ -402,7 +404,7 @@ typedef NS_ENUM(NSInteger, ViewState) {
     for(int i = 0; i < width; ++i) {
         for(int j = 0; j < width; ++j) {
             if(*data & 1) {
-                CGContextSetFillColorWithColor(ctx, [NSColor labelColor].CGColor);
+                CGContextSetFillColorWithColor(ctx, [NSColor blackColor].CGColor);
                 rectDraw.origin = CGPointMake((j + qr_margin) * zoom,(i + qr_margin) * zoom);
                 CGContextAddRect(ctx, rectDraw);
                 CGContextFillPath(ctx);
